
#include "adec.h"
#include <cassert>


ADec::ADec( const ACfg& acfg, std::string& inname )
  : prefix        ( acfg.prefix )
  , V             ( acfg.Vbits  )
  , U             ( acfg.Ubits  )
  , bitstream     ( inname )
  , valid         ( bitstream.isValid() )
  , numSymbols    ( bitstream.getNumSymbols() )
  , numToBeDecoded( numSymbols )
{
  assert( U+V < 64 );

  // init arithmetic coding engine
  {
	  /*if (acfg.version == 0)
	  {
		  alphabetLen = 4;
	  }
	  else if (acfg.version == 2)
	  {
		  alphabetLen = 3;
	  }
	  else
	  {
		  alphabetLen = 256;
	  }
    */
	  A = ((uint64_t)1 << U) - 1;
	  u = 0;
	  uint64_t bit;
	  for (uint64_t i = 0; i < U + V; i++)
	  {
		  bit = bitstream.getBit();
		  u = ((u << 1) ^ bit);
	  }
  }
}


ADec::~ADec()
{
}


uint8_t ADec::decode( const Pmf& pmf )
{
  assert( valid );
  assert( numToBeDecoded );

  // arithmetic decoding
  uint8_t index = 0;
  {
	  for (uint64_t i = 0; true; i++)
	  {
		  if ((A * (pmf[i] + c_v(i, pmf))) > u)
		  {
			  index = i;
			  break;
		  }
	  }

	  A_temp = A * pmf[index];
	  u_temp = u - A * c_v(index, pmf);
	  z = find_z();
	  A = A_temp >> (V - z);
	  uint64_t bit;
	  uint64_t mask = ((uint64_t)1 << (U + V)) - 1;

	  for (uint64_t i = 0; i < z; i++)
	  {
		  bit = bitstream.getBit();
		  u_temp = ((((u_temp << 1) & mask)) ^ bit);
	  }
	  u = u_temp;
  }

  // update symbol count
  numToBeDecoded--;

  if ((numToBeDecoded % 10000) == 0)
  {
	  std::cout << std::to_string(numToBeDecoded) << "\n";
  }

  // return
  return index;
}

uint64_t ADec::c_v(uint8_t index, const Pmf& pmf)
{
	uint64_t c_v_value = 0;
	for (uint64_t k = 0; k < index; k++)
	{
		c_v_value += pmf[k];
	}
	return c_v_value;
}

uint64_t ADec::find_z()
{
	uint64_t max_zeros = U + V;
	for (uint64_t i = 0; i < V + U; i++)
	{
		if (((A_temp >> i) & 1) == 1)
		{
			max_zeros = V + U - (i+1);
		}
	}
	return max_zeros;
}
