
#include <fstream>
#include <iostream>

#include "alphabet.h"
#include "pmf.h"
#include "aenc.h"
#include "encode.h"


bool encode( const ACfg& acfg, std::string& input, std::string& bitstream )
{
  //===== set alphabet and pmf =====
  Alphabet* pAlphabet = 0;
  Pmf*      pPmf      = 0;
  switch( acfg.version )
  {
  case 0:
    std::cout << "Pmf4Fixed used\n";
    pAlphabet = new AlpbabetABCD        ();
    pPmf      = new Pmf4Fixed           ( acfg.Vbits );
    break;
  case 1:
    pAlphabet = new Alphabet256         ();
    pPmf      = new Pmf256Fixed         ( acfg.Vbits );
    break;
  case 2:
	pAlphabet = new AlphabetANB		();
	pPmf	  = new Pmf3Fixed(acfg.Vbits);
	break;
  case 3:
  pAlphabet = new Alphabet256		();
	pPmf	  = new Pmf256Adaptive(acfg.Vbits);
  break;
  default:
    assert(false);
    break;
  }
  Alphabet& alphabet  = *pAlphabet;
  Pmf&      pmf       = *pPmf;

  //===== open input file =====
  std::ifstream infile  ( input, std::ios::binary );
  if( !infile.is_open() )
  {
    std::cerr << "\nERROR: Could not open input file \"" << input << "\"\n";
    return true;
  }

  //===== init arithmetic encoder =====
  AEnc aenc( acfg, bitstream );
  if( !aenc.isValid() )
  {
    std::cerr << "\nERROR: Could not open bitstream file \"" << bitstream << "\"\n";
    return true;
  }

  //===== encode symbols =====
  while( true )
  {
    // read symbol
    int symbol = infile.get();
    if( symbol == std::char_traits<char>::eof() )
    {
      break;
    }

    // encode
    uint8_t index = alphabet.getIndex( uint8_t(symbol) );
    aenc.encode ( index, pmf );
    pmf.update  ( index );
  }
  aenc.terminate();

  //===== delete ======
  delete pAlphabet;
  delete pPmf;

  //===== output and return =====
  std::cout << "\ndone: " << aenc.getNumEncoded() << " symbols encoded.\n\n";
  return false;
}
