
#include "ibitstream.h"
#include <cassert>


IBitstream::IBitstream( std::string& filename )
  : file        ( filename, std::ios::binary )
  , valid       ( file.is_open() )
  , eofread     ( false )
  , numSymbols  ( 0 )
  , nextBits    ( 0 )
  , numNextBits ( 0 )
{
  // read number of coded symbols (first four bytes)
  for( int k = 0; k < 4; k++ )
  {
    int     byte  = file.get();
    assert( byte != std::char_traits<char>::eof() );
    numSymbols   += byte << ( k * 8 );
  }
}


IBitstream::~IBitstream()
{
  if( valid )
  {
    file.close();
  }
}


uint8_t IBitstream::getBit()
{
  assert( valid );

  // read next byte from file
  if( numNextBits == 0 )
  {
    numNextBits = 8;
    nextBits    = 0;
    if( !eofread )
    {
      int byte  = file.get();
      if( byte == std::char_traits<char>::eof() )
      {
        eofread   = true;
        nextBits  = 0;
      }
      else
      {
        nextBits  = uint8_t( byte );
      }
    }
  }

  // extract bit
  uint8_t bit = nextBits >> 7;
  nextBits  <<= 1;
  numNextBits--;
  return  bit;
}

