
#include <cstdlib>
#include <iostream>
#include <string>
#include "acfg.h"
#include "encode.h"
#include "decode.h"
#include "compare.h"


enum OpMode { INVALID, ENCODE, DECODE, COMPARE };
enum AlphabetVersion { v1, v2, invalid};

struct CfgPars
{
  CfgPars() : mode( INVALID ) {}
  OpMode      mode;
  AlphabetVersion aVersion;
  std::string file1;
  std::string file2;
};

bool printUsage( const char* pError = 0 )
{
  if( pError )
  {
    std::cerr << std::endl << "ERROR: " << pError << std::endl;
  }
  std::cout
    << std::endl
    << "Usage:" << std::endl
    << "   encode:   ac -e (orgFile)   (bitstream)" << std::endl
    << "   decode:   ac -d (bitstream) (recFile)"   << std::endl
    << "   compare:  ac -c (orgFile)   (recFile)"   << std::endl
    << std::endl;
  return ( pError != 0 );
}

bool parseCfg( CfgPars& cfgPars, const int argc, const char** argv )
{
  if( argc == 1 )
  {
    return printUsage();
  }
  if (argc != 5)
  {
	  return printUsage("Invalid number of parameters");
  }
  if      ( std::string(argv[1]) == "-e" )  cfgPars.mode = ENCODE;
  else if ( std::string(argv[1]) == "-d" )  cfgPars.mode = DECODE;
  else if ( std::string(argv[1]) == "-c" )  cfgPars.mode = COMPARE;
  else                                      cfgPars.mode = INVALID;
  if( cfgPars.mode == INVALID )
  {
    return printUsage( ( std::string("Invalid operation mode \"") + std::string(argv[1]) + "\"" ).c_str() );
  }
  /*
  if	  ( std::string(argv[2]) == "-v1" ) cfgPars.aVersion = v1;
  else if ( std::string(argv[2]) == "-v2" ) cfgPars.aVersion = v2;
  else                                      cfgPars.aVersion = invalid;
  */
  if (cfgPars.aVersion == invalid)
  {
	  return printUsage( (std::string("Invalid Alphabet version \"") + std::string(argv[2]) + "\"").c_str() );
  }
  cfgPars.file1 = argv[3];
  cfgPars.file2 = argv[4];
  return false;
}


int main( const int argc, const char** argv )
{
  const ACfg acfg(3, false, 31, 31);
  CfgPars     cfgPars;
  if( parseCfg( cfgPars, argc, argv ) )
  {
    return 1;
  }

/*
  //config
  switch ( cfgPars.aVersion )
  {
  case v1:
	  acfg.version = 0;
  case v2:
	  acfg.version = 1;
  default:
	  break;
  }

*/
  // process
  switch( cfgPars.mode )
  {
  case ENCODE:
    if( encode( acfg, cfgPars.file1, cfgPars.file2 ) )
    {
      return 1;
    }
    break;
  case DECODE:
    if( decode( acfg, cfgPars.file1, cfgPars.file2 ) )
    {
      return 1;
    }
    break;
  case COMPARE:
    if( compare( cfgPars.file1, cfgPars.file2 ) )
    {
      return 1;
    }
    break;
  default:
    break;
  }

  return 0;
}
